/*
 * Copyright (c) 2010 ISPRAS
 *
 * Institute for System Programming of the Russian Academy of Sciences (ISPRAS)
 * 25 A.Solzhenitsyn st. Moscow 109004 Russia
 *
 * http://hardware.ispras.ru
 *
 * This code implements working with standard linux context.
 */

#include "sync.h"

#include <assert.h>
#include <stdlib.h>

#include <generator.h>
#include <netndfsm.h>

#if defined(__GNUC__) && !defined(__MINGW32__)
  #include <ucontext.h>
#elif defined(WIN32)
  #include <win32_ucontext.h>
#else
  #error "Unknown platform"
#endif

#ifdef SIGSTKSZ
#undef SIGSTKSZ
#endif
#define SIGSTKSZ        (8192*32)

char stack[SIGSTKSZ];

ucontext_t scenario_context;
ucontext_t testbench_context;

static TestScenario *__localscen;
static TestScenario *__localinitscen;
static int __localargc;
static char **__localargv;
static bool initscenario = false;
static bool netscenario = false;

static int last_operation = 0;

static void scenario_wrapper(void)
{
    if(!netscenario)
        localmain(&TE_NDFSM, __localscen, __localargc, __localargv);
    else
        netmain(&TE_NDFSM, __localscen, __localargc, __localargv);
}

static void scenario_wrapper_with_init(void)
{
    localmain(&TE_NDFSM, __localinitscen, __localargc, __localargv);
    if(!netscenario)
        localmain(&TE_NDFSM, __localscen, __localargc, __localargv);
    else
        netmain(&TE_NDFSM, __localscen, __localargc, __localargv);
}

int start_scenario(TestScenario *scen, int argc, char **argv)
{
    __localscen = scen;
    __localargc = argc;
    __localargv = argv;

    getcontext(&scenario_context);
    scenario_context.uc_link = &testbench_context;
    scenario_context.uc_stack.ss_sp = stack;
    scenario_context.uc_stack.ss_size = SIGSTKSZ;
    makecontext(&scenario_context, initscenario ? scenario_wrapper_with_init : scenario_wrapper, 0);
    swapcontext(&testbench_context, &scenario_context);

    return 0;
}

int start_net_scenario(TestScenario *scen, int argc, char **argv)
{
    netscenario = true;
    return start_scenario(scen, argc, argv);
}

int start_scenario_with_init(TestScenario *initscen, TestScenario *scen, int argc, char **argv)
{
    __localinitscen = initscen;
    initscenario    = true;
    return start_scenario(scen, argc, argv);
}

int start_net_scenario_with_init(TestScenario *initscen, TestScenario *scen, int argc, char **argv)
{
    netscenario = true;
    return start_scenario_with_init(initscen, scen, argc, argv);
}

int end_scenario(void)
{
    return 0;
}

int wait_for_apply(void)
{
    return last_operation;
}

// scenario
void wait_for_check(void)
{
    swapcontext(&scenario_context, &testbench_context);
}

// testbench
void apply_check(void)
{
    swapcontext(&testbench_context, &scenario_context);
}

void apply_finish(void)
{
    last_operation = ACTION_FINISH_OPERATION;
}

void apply_operation(void)
{
    last_operation = ACTION_START_OPERATION;
}

void end_call(void)
{
}
